@@ -353,7 +353,7 @@
     strand <- local({
         X <- ulist(result, "strand", recursive=FALSE)
         values <- do.call(c, lapply(X, as.character))
-        strand(values)
+        factor(values, levels=levels(strand()))
     })
     id <- local({
         X <- ulist(result, "qname")
@@ -390,7 +390,7 @@
         .arg_mismatch_type_err("type", "character(1)")
     if (!missing(filter))
         .check_type_and_length(filter, "SRFilter", NA)
-    vals <- eval(formals(ShortRead:::.readAligned_character)$type)
+    vals <- eval(formals(.readAligned_character)$type)
     if (!type %in% vals)
         .arg_mismatch_value_err("type", type, vals)
     aln <-
